@@ -92,7 +92,7 @@ import Data.Traversable
 #endif
 #if !MIN_VERSION_base(4, 8, 0)
 import Data.Monoid (Monoid(mappend, mempty))
-#elif MIN_VERSION_base(4, 9, 0)
+#elif MIN_VERSION_base(4, 9, 0) && !MIN_VERSION_base(4, 13, 0)
 import Data.Semigroup (Semigroup((<>)))
 #endif
 
@@ -139,7 +139,7 @@ withAsync
   => m a
   -> (Async (StM m a) -> m b)
   -> m b
-withAsync = withAsyncUsing async
+withAsync = liftWithAsync A.withAsync
 {-# INLINABLE withAsync #-}
 
 -- | Generalized version of 'A.withAsyncBound'.
@@ -148,9 +148,17 @@ withAsyncBound
   => m a
   -> (Async (StM m a) -> m b)
   -> m b
-withAsyncBound = withAsyncUsing asyncBound
+withAsyncBound = liftWithAsync A.withAsyncBound
 {-# INLINABLE withAsyncBound #-}
 
+liftWithAsync
+  :: MonadBaseControl IO m
+  => (IO (StM m a) -> (Async (StM m a) -> IO (StM m b)) -> IO (StM m b))
+  -> (m a -> (Async (StM m a) -> m b) -> m b)
+liftWithAsync withA action cont = restoreM =<< do
+  liftBaseWith $ \runInIO -> do
+    withA (runInIO action) (runInIO . cont)
+
 -- | Generalized version of 'A.withAsyncOn'.
 withAsyncOn
   :: MonadBaseControl IO m
